*:************************************************************
*:
*:  Source File: My_box.prg
*:
*:       System: my_box.prg                    
*:       Author: Steven Gibb                   
*:   Copyright (c) 1992, Steven Gibb                   
*:
*:  Comments :
*:
*:  Procs & Funcs: DrawBox()
*:               : ShadowStrip()
*:
*: Documented 10/25/92 at 09:12:43 DOC Vers 2.12, Source Code Documenter
*:************************************************************
/*

DrawBox()        Draws a box on screen.

Syntax

   DrawBox(<iTrow>,<iLcol>,<iBrow>,<iRcol>,[iBorder],[iShadow],[cColor];
             [cBoxTitle], [cTitleColor])

     Arguments    <> required  [] optional

    <iTrow>       Top row of box
    <iLcol>       Left column of box
    <iBrow>       Bottom row of box
    <iRcol>       Right col of box

    [iBorder]     If NIL border = space, if 1 border = , else border = 
    [iShadow]     If !NIL and box not on screen edge box shadow is displayed
    [cColor]      If NIL current color is used, else cColor is used for box
                  and old color is restored upon exit from function
    [cBoxTitle]   If !NIL displays a title on the top line of the box
    [cTitleColor] If !NIL changes the color of the title


 Returns

    NIL

 Description

    Draws retangular area on screen.  Area can have no border, single
    or double line border.  Color can be current color or new color.
    Old color is restored upon exit from function.  Box is filled
    with SPACE(1).  Box may use shadow option.  Direction shadow is
    cast from box is controlled by uncommenting one of the four
    #define statements in function source code.


*/



*!************************************************************
*!  
*!             FUNCTION DrawBox
*!
*!     Calls : CHR()                 (function in ???)
*!           : LEN()                 (function in ???)
*!           : MAXCOL()              (function in ???)
*!           : SETCOLOR()            (function in ???)
*!           : ShadowStrip()    
*!           : SPACE()               (function in ???)
*!           : SUBSTR()              (function in ???)
*!           : VALTYPE()             (function in ???)
*!
*!   Purpose :
*!
*!   In      : trow
*!           : lcol
*!           : brow
*!           : rcol
*!           : border_char
*!           : shadow
*!           : cColor
*!           : cBoxTitle
*!           : cTitleColor
*!
*!   Out     :
*!
*!   Assumes : parameters are  correct data type
*!
*!************************************************************
FUNCTION DrawBox( trow, lcol, brow,   rcol,;
       border_char, shadow, cColor, cBoxTitle, cTitleColor)
  
  LOCAL cOldColor  := SETCOLOR()
  LOCAL iBoxWidth  := rcol - lcol
  LOCAL iBoxHeight := brow - trow
  LOCAL cLeftEnd, cRightEnd
  LOCAL cTitle
  
  //   only 1 of next 4 #defines can be uncommented
  #define LOWER_LEFT
  //    #define UPPER_LEFT
  //    #define UPPER_RIGHT
  //    #define LOWER_RIGHT
  
  #ifdef LOWER_LEFT
    LOCAL iShadRow      := brow + 1
    LOCAL iShadCol      := lcol - 2
    LOCAL iTopVertShad  := trow + 1
    LOCAL iLeftHorzShad := lcol - 2
  #endif
  
  #ifdef UPPER_LEFT
    LOCAL iShadRow      := trow - 1
    LOCAL iShadCol      := lcol - 2
    LOCAL iTopVertShad  := trow - 1
    LOCAL iLeftHorzShad := lcol - 2
  #endif
  
  #ifdef UPPER_RIGHT
    LOCAL iShadRow      := trow - 1
    LOCAL iShadCol      := rcol + 1
    LOCAL iTopVertShad  := trow - 1
    LOCAL iLeftHorzShad := lcol + 2
  #endif
  
  #ifdef LOWER_RIGHT
    LOCAL iShadRow      := brow + 1
    LOCAL iShadCol      := rcol + 1
    LOCAL iTopVertShad  := trow + 1
    LOCAL iLeftHorzShad := lcol + 2
  #endif
  
  IF cColor <> NIL
    SETCOLOR( cColor )
  ENDIF
  IF cTitleColor = NIL
    cTitleColor := SETCOLOR()
  ENDIF
  
  #define SINGLE_LINE  ( CHR(218) + CHR(196) ;
       + CHR(191) + CHR(179) + CHR(217) ;
       + CHR(196) + CHR(192) + CHR(179) ;
       + CHR(32) )
  
  #define DOUBLE_LINE  ( CHR(201) + CHR(205) ;
       + CHR(187) + CHR(186) + CHR(188) ;
       + CHR(205) + CHR(200) + CHR(186) ;
       + CHR(32))
  
  #define SINGLE_LEFT_END   CHR(180)
  #define SINGLE_RIGHT_END  CHR(195)
  
  #define DBL_LEFT_END      CHR(181)
  #define DBL_RIGHT_END     CHR(198)
  
  
  
  DO CASE
  CASE border_char = NIL
    border_char := SPACE(9)
    cLeftEnd    := SPACE(1)
    cRightEnd   := SPACE(1)
  CASE border_char = 1
    border_char := SINGLE_LINE
    cLeftEnd    := SINGLE_LEFT_END
    cRightEnd   := SINGLE_RIGHT_END
  OTHERWISE
    border_char := DOUBLE_LINE
    cLeftEnd    := DBL_LEFT_END
    cRightEnd   := DBL_RIGHT_END
  ENDCASE
  
  @ trow,lcol,brow,rcol BOX border_char
  IF cBoxTitle <> NIL .AND. VALTYPE( cBoxTitle ) = [C]
    cTitle := SUBSTR( cBoxTitle,1,( rcol - lcol - 7) )
    @ trow, lCol+2 SAY cLeftEnd + SPACE(2+ LEN(cTitle) ) + cRightEnd
    @ trow, lCol+4 SAY cTitle COLOR cTitleColor
    
  ENDIF
  
  IF shadow <> NIL .AND. ;
         lcol > 2 .AND. rcol < MAXCOL()
    
    // vertical shadow
    ShadowStrip( iTopVertShad,           ;
                 iShadCol  ,             ;
                 iTopVertShad+iBoxHeight,;
                 iShadCol+1 )
    
    // horizontal shadow
    ShadowStrip( iShadRow,               ;
                 iLeftHorzShad,          ;
                 iShadRow,               ;
                 iLeftHorzShad+iBoxWidth)
    
  ENDIF
  
  SETCOLOR( cOldColor )
  
RETURN( NIL )




*!************************************************************
*!  
*!             FUNCTION ShadowStrip
*!
*! Called by : DrawBox        
*!
*!     Calls : CHR()                 (function in ???)
*!           : LEN()                 (function in ???)
*!           : REPLICATE()           (function in ???)
*!           : RESTSCREEN()          (function in ???)
*!           : SAVESCREEN()          (function in ???)
*!           : TRANSFORM()           (function in ???)
*!
*!   Purpose :
*!
*!   In      : t
*!           : l
*!           : b
*!           : r
*!
*!   Out     :
*!
*!   Assumes :
*!
*!************************************************************
FUNCTION ShadowStrip( t, l, b, r )
  LOCAL cStrip := SAVESCREEN( t, l, b, r )
  RESTSCREEN( t, l, b, r, ;
         TRANSFORM( cStrip, REPLICATE( "X" + CHR(8), LEN( cStrip )/2 ) )  )
RETURN( NIL )


*:   EOF: My_box.prg
